pub struct MemLevel(/* private fields */);Expand description
Location in the memory hierarchy.
This is used by DataSource.
Implementations
Sourceimpl MemLevel
impl MemLevel
pub const NA: MemLevel
pub const HIT: MemLevel
pub const MISS: MemLevel
pub const L1: MemLevel
pub const LFB: MemLevel
pub const L2: MemLevel
pub const L3: MemLevel
pub const LOC_RAM: MemLevel
pub const REM_RAM1: MemLevel
pub const REM_RAM2: MemLevel
pub const REM_CCE1: MemLevel
pub const REM_CCE2: MemLevel
pub const IO: MemLevel
pub const UNC: MemLevel
Sourceimpl MemLevel
impl MemLevel
Sourcepub const fn bits(&self) -> u64
pub const fn bits(&self) -> u64
Get the underlying bits value.
The returned value is exactly the bits set in this flags value.
Sourcepub const fn from_bits(bits: u64) -> Option<MemLevel>
pub const fn from_bits(bits: u64) -> Option<MemLevel>
Convert from a bits value.
This method will return None if any unknown bits are set.
Sourcepub const fn from_bits_truncate(bits: u64) -> MemLevel
pub const fn from_bits_truncate(bits: u64) -> MemLevel
Convert from a bits value, unsetting any unknown bits.
Sourcepub const fn from_bits_retain(bits: u64) -> MemLevel
pub const fn from_bits_retain(bits: u64) -> MemLevel
Convert from a bits value exactly.
Sourcepub fn from_name(name: &str) -> Option<MemLevel>
pub fn from_name(name: &str) -> Option<MemLevel>
Get a flags value with the bits of a flag with the given name set.
This method will return None if name is empty or doesn’t
correspond to any named flag.
Sourcepub const fn intersects(&self, other: MemLevel) -> bool
pub const fn intersects(&self, other: MemLevel) -> bool
Whether any set bits in a source flags value are also set in a target flags value.
Sourcepub const fn contains(&self, other: MemLevel) -> bool
pub const fn contains(&self, other: MemLevel) -> bool
Whether all set bits in a source flags value are also set in a target flags value.
Sourcepub fn remove(&mut self, other: MemLevel)
pub fn remove(&mut self, other: MemLevel)
The intersection of a source flags value with the complement of a target flags value (&!).
This method is not equivalent to self & !other when other has unknown bits set.
remove won’t truncate other, but the ! operator will.
Sourcepub fn toggle(&mut self, other: MemLevel)
pub fn toggle(&mut self, other: MemLevel)
The bitwise exclusive-or (^) of the bits in two flags values.
Sourcepub fn set(&mut self, other: MemLevel, value: bool)
pub fn set(&mut self, other: MemLevel, value: bool)
Call insert when value is true or remove when value is false.
Sourcepub const fn intersection(self, other: MemLevel) -> MemLevel
pub const fn intersection(self, other: MemLevel) -> MemLevel
The bitwise and (&) of the bits in two flags values.
Sourcepub const fn union(self, other: MemLevel) -> MemLevel
pub const fn union(self, other: MemLevel) -> MemLevel
The bitwise or (|) of the bits in two flags values.
Sourcepub const fn difference(self, other: MemLevel) -> MemLevel
pub const fn difference(self, other: MemLevel) -> MemLevel
The intersection of a source flags value with the complement of a target flags value (&!).
This method is not equivalent to self & !other when other has unknown bits set.
difference won’t truncate other, but the ! operator will.
Sourcepub const fn symmetric_difference(self, other: MemLevel) -> MemLevel
pub const fn symmetric_difference(self, other: MemLevel) -> MemLevel
The bitwise exclusive-or (^) of the bits in two flags values.
Sourcepub const fn complement(self) -> MemLevel
pub const fn complement(self) -> MemLevel
The bitwise negation (!) of the bits in a flags value, truncating the result.
Sourceimpl MemLevel
impl MemLevel
Sourcepub const fn iter(&self) -> Iter<MemLevel>
pub const fn iter(&self) -> Iter<MemLevel>
Yield a set of contained flags values.
Each yielded flags value will correspond to a defined named flag. Any unknown bits will be yielded together as a final flags value.
Sourcepub const fn iter_names(&self) -> IterNames<MemLevel>
pub const fn iter_names(&self) -> IterNames<MemLevel>
Yield a set of contained named flags values.
This method is like iter, except only yields bits in contained named flags.
Any unknown bits, or bits not corresponding to a contained flag will not be yielded.