pub struct ReadFormat(/* private fields */);Expand description
Flags that control what data is returned when reading from a perf_event file descriptor.
See the man page for the authoritative documentation on what these flags do.
Implementations
Sourceimpl ReadFormat
impl ReadFormat
Sourcepub const TOTAL_TIME_ENABLED: ReadFormat
pub const TOTAL_TIME_ENABLED: ReadFormat
Emit the total amount of time the counter has spent enabled.
Sourcepub const TOTAL_TIME_RUNNING: ReadFormat
pub const TOTAL_TIME_RUNNING: ReadFormat
Emit the total amount of time the counter was actually on the CPU.
Sourcepub const ID: ReadFormat
pub const ID: ReadFormat
Emit the counter ID.
Sourcepub const GROUP: ReadFormat
pub const GROUP: ReadFormat
If in a group, read all the counters in the group at once.
Sourcepub const LOST: ReadFormat
pub const LOST: ReadFormat
Emit the number of lost samples for this event.
Sourceimpl ReadFormat
impl ReadFormat
Sourcepub const fn empty() -> ReadFormat
pub const fn empty() -> ReadFormat
Get a flags value with all bits unset.
Sourcepub const fn all() -> ReadFormat
pub const fn all() -> ReadFormat
Get a flags value with all known bits set.
Sourcepub const fn bits(&self) -> u64
pub const fn bits(&self) -> u64
Get the underlying bits value.
The returned value is exactly the bits set in this flags value.
Sourcepub const fn from_bits(bits: u64) -> Option<ReadFormat>
pub const fn from_bits(bits: u64) -> Option<ReadFormat>
Convert from a bits value.
This method will return None if any unknown bits are set.
Sourcepub const fn from_bits_truncate(bits: u64) -> ReadFormat
pub const fn from_bits_truncate(bits: u64) -> ReadFormat
Convert from a bits value, unsetting any unknown bits.
Sourcepub const fn from_bits_retain(bits: u64) -> ReadFormat
pub const fn from_bits_retain(bits: u64) -> ReadFormat
Convert from a bits value exactly.
Sourcepub fn from_name(name: &str) -> Option<ReadFormat>
pub fn from_name(name: &str) -> Option<ReadFormat>
Get a flags value with the bits of a flag with the given name set.
This method will return None if name is empty or doesn’t
correspond to any named flag.
Sourcepub const fn intersects(&self, other: ReadFormat) -> bool
pub const fn intersects(&self, other: ReadFormat) -> bool
Whether any set bits in a source flags value are also set in a target flags value.
Sourcepub const fn contains(&self, other: ReadFormat) -> bool
pub const fn contains(&self, other: ReadFormat) -> bool
Whether all set bits in a source flags value are also set in a target flags value.
Sourcepub fn insert(&mut self, other: ReadFormat)
pub fn insert(&mut self, other: ReadFormat)
The bitwise or (|) of the bits in two flags values.
Sourcepub fn remove(&mut self, other: ReadFormat)
pub fn remove(&mut self, other: ReadFormat)
The intersection of a source flags value with the complement of a target flags value (&!).
This method is not equivalent to self & !other when other has unknown bits set.
remove won’t truncate other, but the ! operator will.
Sourcepub fn toggle(&mut self, other: ReadFormat)
pub fn toggle(&mut self, other: ReadFormat)
The bitwise exclusive-or (^) of the bits in two flags values.
Sourcepub fn set(&mut self, other: ReadFormat, value: bool)
pub fn set(&mut self, other: ReadFormat, value: bool)
Call insert when value is true or remove when value is false.
Sourcepub const fn intersection(self, other: ReadFormat) -> ReadFormat
pub const fn intersection(self, other: ReadFormat) -> ReadFormat
The bitwise and (&) of the bits in two flags values.
Sourcepub const fn union(self, other: ReadFormat) -> ReadFormat
pub const fn union(self, other: ReadFormat) -> ReadFormat
The bitwise or (|) of the bits in two flags values.
Sourcepub const fn difference(self, other: ReadFormat) -> ReadFormat
pub const fn difference(self, other: ReadFormat) -> ReadFormat
The intersection of a source flags value with the complement of a target flags value (&!).
This method is not equivalent to self & !other when other has unknown bits set.
difference won’t truncate other, but the ! operator will.
Sourcepub const fn symmetric_difference(self, other: ReadFormat) -> ReadFormat
pub const fn symmetric_difference(self, other: ReadFormat) -> ReadFormat
The bitwise exclusive-or (^) of the bits in two flags values.
Sourcepub const fn complement(self) -> ReadFormat
pub const fn complement(self) -> ReadFormat
The bitwise negation (!) of the bits in a flags value, truncating the result.
Sourceimpl ReadFormat
impl ReadFormat
Sourcepub const fn iter(&self) -> Iter<ReadFormat> ⓘ
pub const fn iter(&self) -> Iter<ReadFormat> ⓘ
Yield a set of contained flags values.
Each yielded flags value will correspond to a defined named flag. Any unknown bits will be yielded together as a final flags value.
Sourcepub const fn iter_names(&self) -> IterNames<ReadFormat> ⓘ
pub const fn iter_names(&self) -> IterNames<ReadFormat> ⓘ
Yield a set of contained named flags values.
This method is like iter, except only yields bits in contained named flags.
Any unknown bits, or bits not corresponding to a contained flag will not be yielded.