pub struct SampleFlags(/* private fields */);Expand description
Specifies which fields to include in the sample.
These values correspond to PERF_SAMPLE_x values. See the
manpage for documentation on what they mean.
Implementations
Sourceimpl SampleFlags
impl SampleFlags
pub const IP: SampleFlags
pub const TID: SampleFlags
pub const TIME: SampleFlags
pub const ADDR: SampleFlags
pub const READ: SampleFlags
pub const CALLCHAIN: SampleFlags
pub const ID: SampleFlags
pub const CPU: SampleFlags
pub const PERIOD: SampleFlags
pub const STREAM_ID: SampleFlags
pub const RAW: SampleFlags
pub const BRANCH_STACK: SampleFlags
pub const REGS_USER: SampleFlags
pub const STACK_USER: SampleFlags
pub const WEIGHT: SampleFlags
pub const DATA_SRC: SampleFlags
pub const IDENTIFIER: SampleFlags
pub const TRANSACTION: SampleFlags
pub const REGS_INTR: SampleFlags
pub const PHYS_ADDR: SampleFlags
pub const AUX: SampleFlags
pub const CGROUP: SampleFlags
pub const DATA_PAGE_SIZE: SampleFlags
pub const CODE_PAGE_SIZE: SampleFlags
pub const WEIGHT_STRUCT: SampleFlags
Sourceimpl SampleFlags
impl SampleFlags
Sourcepub const fn empty() -> SampleFlags
pub const fn empty() -> SampleFlags
Get a flags value with all bits unset.
Sourcepub const fn all() -> SampleFlags
pub const fn all() -> SampleFlags
Get a flags value with all known bits set.
Sourcepub const fn bits(&self) -> u64
pub const fn bits(&self) -> u64
Get the underlying bits value.
The returned value is exactly the bits set in this flags value.
Sourcepub const fn from_bits(bits: u64) -> Option<SampleFlags>
pub const fn from_bits(bits: u64) -> Option<SampleFlags>
Convert from a bits value.
This method will return None if any unknown bits are set.
Sourcepub const fn from_bits_truncate(bits: u64) -> SampleFlags
pub const fn from_bits_truncate(bits: u64) -> SampleFlags
Convert from a bits value, unsetting any unknown bits.
Sourcepub const fn from_bits_retain(bits: u64) -> SampleFlags
pub const fn from_bits_retain(bits: u64) -> SampleFlags
Convert from a bits value exactly.
Sourcepub fn from_name(name: &str) -> Option<SampleFlags>
pub fn from_name(name: &str) -> Option<SampleFlags>
Get a flags value with the bits of a flag with the given name set.
This method will return None if name is empty or doesn’t
correspond to any named flag.
Sourcepub const fn intersects(&self, other: SampleFlags) -> bool
pub const fn intersects(&self, other: SampleFlags) -> bool
Whether any set bits in a source flags value are also set in a target flags value.
Sourcepub const fn contains(&self, other: SampleFlags) -> bool
pub const fn contains(&self, other: SampleFlags) -> bool
Whether all set bits in a source flags value are also set in a target flags value.
Sourcepub fn insert(&mut self, other: SampleFlags)
pub fn insert(&mut self, other: SampleFlags)
The bitwise or (|) of the bits in two flags values.
Sourcepub fn remove(&mut self, other: SampleFlags)
pub fn remove(&mut self, other: SampleFlags)
The intersection of a source flags value with the complement of a target flags value (&!).
This method is not equivalent to self & !other when other has unknown bits set.
remove won’t truncate other, but the ! operator will.
Sourcepub fn toggle(&mut self, other: SampleFlags)
pub fn toggle(&mut self, other: SampleFlags)
The bitwise exclusive-or (^) of the bits in two flags values.
Sourcepub fn set(&mut self, other: SampleFlags, value: bool)
pub fn set(&mut self, other: SampleFlags, value: bool)
Call insert when value is true or remove when value is false.
Sourcepub const fn intersection(self, other: SampleFlags) -> SampleFlags
pub const fn intersection(self, other: SampleFlags) -> SampleFlags
The bitwise and (&) of the bits in two flags values.
Sourcepub const fn union(self, other: SampleFlags) -> SampleFlags
pub const fn union(self, other: SampleFlags) -> SampleFlags
The bitwise or (|) of the bits in two flags values.
Sourcepub const fn difference(self, other: SampleFlags) -> SampleFlags
pub const fn difference(self, other: SampleFlags) -> SampleFlags
The intersection of a source flags value with the complement of a target flags value (&!).
This method is not equivalent to self & !other when other has unknown bits set.
difference won’t truncate other, but the ! operator will.
Sourcepub const fn symmetric_difference(self, other: SampleFlags) -> SampleFlags
pub const fn symmetric_difference(self, other: SampleFlags) -> SampleFlags
The bitwise exclusive-or (^) of the bits in two flags values.
Sourcepub const fn complement(self) -> SampleFlags
pub const fn complement(self) -> SampleFlags
The bitwise negation (!) of the bits in a flags value, truncating the result.
Sourceimpl SampleFlags
impl SampleFlags
Sourcepub const fn iter(&self) -> Iter<SampleFlags> ⓘ
pub const fn iter(&self) -> Iter<SampleFlags> ⓘ
Yield a set of contained flags values.
Each yielded flags value will correspond to a defined named flag. Any unknown bits will be yielded together as a final flags value.
Sourcepub const fn iter_names(&self) -> IterNames<SampleFlags> ⓘ
pub const fn iter_names(&self) -> IterNames<SampleFlags> ⓘ
Yield a set of contained named flags values.
This method is like iter, except only yields bits in contained named flags.
Any unknown bits, or bits not corresponding to a contained flag will not be yielded.