Hardware

Enum Hardware 

#[repr(u32)]
pub enum Hardware { CPU_CYCLES = 0, INSTRUCTIONS = 1, CACHE_REFERENCES = 2, CACHE_MISSES = 3, BRANCH_INSTRUCTIONS = 4, BRANCH_MISSES = 5, BUS_CYCLES = 6, STALLED_CYCLES_FRONTEND = 7, STALLED_CYCLES_BACKEND = 8, REF_CPU_CYCLES = 9, }
Hardware counters.

These are counters implemented by the processor itself. Such counters vary from one architecture to the next, and even different models within a particular architecture will often change the way they expose this data. This is a selection of portable names for values that can be obtained on a wide variety of systems.

Each variant of this enum corresponds to a particular PERF_COUNT_HW_… value supported by the perf_event_open system call.

Variants



CPU_CYCLES = 0

Total cycles.



INSTRUCTIONS = 1

Retired instructions.



CACHE_REFERENCES = 2

Cache accesses.



CACHE_MISSES = 3

Cache misses.



BRANCH_INSTRUCTIONS = 4

Retired branch instructions.



BRANCH_MISSES = 5

Mispredicted branch instructions.



BUS_CYCLES = 6

Bus cycles.



STALLED_CYCLES_FRONTEND = 7

Stalled cycles during issue.



STALLED_CYCLES_BACKEND = 8

Stalled cycles during retirement.



REF_CPU_CYCLES = 9

Total cycles, independent of frequency scaling.