Software

Struct Software 

#[repr(transparent)]
pub struct Software(pub u64);
Software counters, implemented by the kernel.

Each variant of this enum corresponds to a particular PERF_COUNT_SW_… value supported by the perf_event_open system call.

Tuple Fields

0: u64

Implementations

Source

impl Software

pub const CPU_CLOCK: Self

High-resolution per-CPU timer.

pub const TASK_CLOCK: Self

Per-task clock count.

pub const PAGE_FAULTS: Self

Page faults.

pub const CONTEXT_SWITCHES: Self

Context switches.

pub const CPU_MIGRATIONS: Self

Process migration to another CPU.

pub const PAGE_FAULTS_MIN: Self

Minor page faults: resolved without needing I/O.

pub const PAGE_FAULTS_MAJ: Self

Major page faults: I/O was required to resolve these.

pub const ALIGNMENT_FAULTS: Self

Alignment faults that required kernel intervention.

This is only generated on some CPUs, and never on x86_64 or ARM.

pub const EMULATION_FAULTS: Self

Instruction emulation faults.

pub const DUMMY: Self

Placeholder, for collecting informational sample records.

pub const BPF_OUTPUT: Self

Special event type for streaming data from a eBPF program.

See the documentation of the bpf_perf_event_output method in the bpf-helpers(7) manpage for details on how to use this event type.

pub const CGROUP_SWITCHES: Self

Context switches to a task in a different cgroup.