#[repr(u32)]pub enum WhichCache {
L1D = 0,
L1I = 1,
LL = 2,
DTLB = 3,
ITLB = 4,
BPU = 5,
NODE = 6,
}Expand description
A cache whose events we would like to count.
This is used in the Cache type as part of the identification of a cache
event. Each variant here corresponds to a particular
PERF_COUNT_HW_CACHE_... constant supported by the perf_event_open
system call.
Variants
L1D = 0
Level 1 data cache.
L1I = 1
Level 1 instruction cache.
LL = 2
Last-level cache.
DTLB = 3
Data translation lookaside buffer (virtual address translation).
ITLB = 4
Instruction translation lookaside buffer (virtual address translation).
BPU = 5
Branch prediction.
NODE = 6
Memory accesses that stay local to the originating NUMA node.