#[repr(u32)]pub enum WhichCache {
L1D = 0,
L1I = 1,
LL = 2,
DTLB = 3,
ITLB = 4,
BPU = 5,
NODE = 6,
}Expand description
A cache whose events we would like to count.
This is used in the Cache type as part of the identification of a cache
event. Each variant here corresponds to a particular
PERF_COUNT_HW_CACHE_... constant supported by the perf_event_open
system call.
Variants
L1D = 0
for measuring Level 1 Data Cache
L1I = 1
for measuring Level 1 Instruction Cache
LL = 2
for measuring Last-Level Cache
DTLB = 3
for measuring the Data TLB
ITLB = 4
for measuring the Instruction TLB
BPU = 5
for measuring the branch prediction unit
NODE = 6
(since Linux 3.1) for measuring local memory accesses