RealHooks

Struct RealHooks 

pub struct RealHooks;
An implementation of the Hooks trait in terms of the real Linux system calls.

This type implements each methods of the Hooks trait by calling the underlying system call or ioctl. The following call is equivalent to calling clear_thread_hooks:

unsafe {
    set_thread_hooks(Box::new(RealHooks));
}

If what you want is non-intercepted access to the underlying system calls, it’s probably better to just access the perf_event_open_sys crate directly, rather than using this type.